package game.command;

import java.util.Collection;
import java.util.Map;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import game.command.DoubleParser.DefType;
import game.world.World;

public abstract class ScriptExecutable implements Executable {
	private final String name;
	private final Map<String, Parameter> parameters = Maps.newHashMap();
	
	private int parPos = 0;
	private boolean parReq = true;
	
	protected ScriptExecutable(String name) {
		this.name = name;
	}
	
	protected ScriptExecutable setParamsOptional() {
		this.parReq = false;
		return this;
	}
	
	protected ScriptExecutable addParameter(String name, ArgumentParser ... parsers) {
		this.parameters.put(name, new Parameter(name, (char)0, this.parPos++, this.parReq, Lists.newArrayList(parsers)));
		return this;
	}
	
	protected ScriptExecutable addParameter(String name, char shortName, ArgumentParser ... parsers) {
		this.parameters.put(name, new Parameter(name, shortName, -1, false, Lists.newArrayList(parsers)));
		return this;
	}
	
	protected ScriptExecutable addParameter(ArgumentParser parser) {
		return this.addParameter(parser.getName(), parser);
	}
	
	protected ScriptExecutable addParameter(char shortName, ArgumentParser parser) {
		return this.addParameter(parser.getName(), shortName, parser);
	}
	
	protected ScriptExecutable addVector(String name, boolean defaulted) {
		return this.addParameter(name, new DoubleParser("x", defaulted ? DefType.X : null, (double)(-World.MAX_SIZE), (double)World.MAX_SIZE), new DoubleParser("y", defaulted ? DefType.Y : null, 0.0, (double)World.HEIGHT), new DoubleParser("z", defaulted ? DefType.Z : null, (double)(-World.MAX_SIZE), (double)World.MAX_SIZE));
	}
	
	protected ScriptExecutable addWorld(String name, boolean defaulted) {
		return this.addParameter(new WorldParser(name, false, defaulted));
	}
	
	protected ScriptExecutable addInt(String name, char shortName, int min, int max, int def) {
		return this.addParameter(shortName, new IntParser(name, false, def, min, max, def));
	}
	
	protected ScriptExecutable addInt(String name, int min, int max, int def) {
		return this.addParameter(new IntParser(name, false, def, min, max, def));
	}
	
	protected ScriptExecutable addFlag(String name, char shortName) {
		return this.addParameter(name, shortName);
	}
	
	protected ScriptExecutable addFlag(String name) {
		return this.addParameter(name, name.charAt(0));
	}
	
	protected ScriptExecutable addEnum(String name, Object ... values) {
		return this.addParameter(new EnumParser(name, null, values));
	}
	
	protected ScriptExecutable addEnumDef(String name, Object def, Object ... values) {
		return this.addParameter(new EnumParser(name, null, values));
	}
	
	protected ScriptExecutable addEnum(String name, Collection<?> values) {
		return this.addEnum(name, (Object[])values.toArray(new String[values.size()]));
	}
	
	protected ScriptExecutable addEnumDef(String name, Object def, Collection<?> values) {
		return this.addEnumDef(name, def, (Object[])values.toArray(new String[values.size()]));
	}
	
	protected ScriptExecutable addTag(String name) {
		return this.addParameter(new TagParser(name, null));
	}
	
	protected ScriptExecutable addTag(String name, char shortName) {
		return this.addParameter(shortName, new TagParser(name, null));
	}
	
	public Object exec(ScriptEnvironment env, ScriptArgs args) {
		return null;
	}

	public Map<String, Parameter> getParameters() {
		return this.parameters;
	}

	public String getName() {
		return this.name;
	}
}
